SEXP_CONV_PATH "Goba"

open VecOps

type init = { init_at : float * float;
	      init_angle : float;
	      init_parent : Object.id; 
	      init_length : int; }
with sexp

let recalc_distance_interval = 0.4

let laser_step = 10

let mask = CollisionMask.circle_loc_mask 5

let explosion_interval = 1.0 /. 20.0

class t id init0 =
  let init = Object.default_with init0 init_of_sexp in
      
object (self)
  inherit Object.t id as super

  val mutable recalc_distance = 0.0
  val mutable time_left = 5.0
  val mutable length = init.init_length
  val mutable damage_points = 0.0
  val mutable explosion_time = 0.0
  val mutable alive = true

  method get_type = ObjectType.Laser

  method collision_area =
    ((0.0, 0.0), (0.0, 0.0)), CollisionMask.empty_mask
      
  method is_alive = alive

  method location = init.init_at
  method speed = 0.0, 0.0

  method get_state delay =
    Object.sexp_of_state sexp_of_init
      { Object.state_super = super#get_state delay;
	state_init = { init with init_length = length } }

  method set_state state delay =
    let state = Object.state_of_sexp init_of_sexp state in
      super#set_state state.Object.state_super delay

  (* this object is very much static *)
  method set_location _ = ()
  method set_speed _ = ()

  method tick delta world = 
    if alive then
      begin
	if time_left < 0.0 then
	  begin
	    alive <- false;
	    self#change;
	  end
	else
	  begin
	    recalc_distance <- recalc_distance -. delta;
	    time_left <- time_left -. delta;
	    damage_points <- damage_points +. delta *. 150.0;
	    explosion_time <- explosion_time -. delta;
	    if damage_points > 1.0 then
	      let dir = (cos init.init_angle, -. sin init.init_angle) in
	      let rec hit dist prev_objs =
		if dist < 50000 then
		  begin
		    let at = init.init_at +|. Vector.v2 (float dist) *|. dir in
		      match 
			match prev_objs with 
			  | None -> 
			      world#objs 
				(at -|. (70.0, 70.0), at +|. (55.0, 55.0)) 
				(fun o -> 
				   let objid = o#get_id in
				     List.for_all (fun id -> objid <> id) [init.init_parent;self#get_id] && o#destructible &&
				       let (at1, mask1) = mask in
				       let at1 = at1 +| Vector.int2 at in
				       let ((at2, _), mask2) = o#collision_area in
				       let at2 = Vector.int2 at2 in
					 (*				   Printf.printf "Comparing masks at %s and %s (%s)\n%!" 
									   (Vector.i_to_string at1)
									   (Vector.i_to_string at2)
									   (Object.string_of_id objid);*)
					 (*				   object_collides 
									   o
									   (fun () ->*)(
					   CollisionMask.collides 
					     (at1, mask1) 
					     (at2, mask2))
				)
			  | Some objs -> objs with
			      | [] -> hit (dist + laser_step) None
			      | objs -> 
				  (*		Printf.printf "Impact oids %s!\n%!"
						(String.concat " "
						(List.map
						(fun o -> Object.string_of_id (o#get_id))
						objs
						));*)
				  if prev_objs = None && explosion_time < 0.0 then
				    begin
				      world#explode ~sound:false at (0.0, 0.0);
				      explosion_time <- explosion_time +. explosion_interval;
				    end;
				  damage_points <- 
				    List.fold_left (
				      fun damage_left o -> 
					if damage_left > 1.0 then
					  begin
					    (*							 Printf.printf "Hep for %s\n%!" (Object.string_of_id o#get_id);*)
					    o#impact world 1;
					    damage_left -. 1.0
					  end
					else
					  damage_left
				    ) 
				    damage_points objs;
				  if damage_points > 1.0 then
				    hit dist (Some objs)
				  else
				    dist
		  end
		else
		  dist
	      in
	      let recalc = recalc_distance < 0.0 in
		length <- hit (if recalc then
				 0
			       else
				 length) None;
		if recalc then recalc_distance <- recalc_distance_interval
	  end
      end

  method z_location = -5000.0

  method render gfx _ ofs = 
    if self#is_alive then
      (*    let ang = (sin (time /. 10.0) *. (GobaMath.pi /. 2.0)) in)*)
      (*let ang = time in*)
      let ang = init.init_angle in
      let base = Vector.base (cos ang, sin ang) in
      let clip = gfx#get_clip_rect in
      let color = GobaMath.ipolate3 (Random.float 1.0) (255, 0, 0) (255, 255, 0) in
	Polygon.polygon 
	  (Polygon.clip_rect
	     (List.map (fun v -> Vector.int2 (v **|. base) +| ofs) [(0.0, -3.0); (0.0, 3.0); (float length, 3.0); (float length, -3.0)])
	     clip)
	  (fun row x0 x1 -> 
	     gfx#rectangle color (x0, row) (x1 - x0 + 1, 1))

  initializer
    (match init0 with 
       | Object.Init _ -> ()
       | Object.State (state0, age) -> self#set_state state0 age);
    self#change;
end
